#!/usr/bin/env python
# Copyright 2008-2009 David D Lowe
# released under the GPLv3

from pigment import PigmentType, Pigment
from pigment_more import DirBasedPigment, DirBasedPigmentType
import pigment
import os, sys
import gtk
from glob import glob
if __name__ == "__main__" or 'pychecker' in sys.argv[0]:
    sys.path.append("..")
import handy
import ep_exceptions
import hashlib
from gettext import gettext as _


class GDMType(DirBasedPigmentType):
    humanName = {"en":"GDM"}
    humanPluralName = {"en":"GDM"}
    codeName = "gdm"
    abbreviation = "gdm"
    directoryName = "gdm"
    canGeneratePreview = True

    attachmentTypeHelp = _("The theme file should be the directory that bears the name of theme, and containing a file named GdmGreeterTheme.desktop")
    
    def get_Pigment(self):
        return GDM
        
    publicBaseDir = "/usr/share/gdm/themes"
    personalBaseDir = ""
    
    isDirectoryShared = False
    specificChildren = ()
    
class GDM(DirBasedPigment):
    
    
    def __init__(self):
        DirBasedPigment.__init__(self)
        self.type = GDMType()
    
    def activate(self, progressCallback=None, shell=None):
        if progressCallback is None:
            progressCallback = lambda(a1): False
        if shell is None:
            print >> sys.stderr, "shell is None, should include a shell with root acces for GDM"
            import shell
            shell = shell.SubprocessShell()
            shell.prepare()
        if self.attDirname == "" or self.attDirname is None:
            print >> sys.stderr, "GDM object (" + str(self) + ") has no attDirname"
            raise(Exception("GDM object has no attDirname"))
        if progressCallback(0.0): return
        from ConfigParser import SafeConfigParser # for reading INI files
        class BetterConfigParser(SafeConfigParser):
            """Better than SafeConfigParser: it's case-sensitive"""
            def __init__(self):
                SafeConfigParser.__init__(self)
            def optionxform(self, optionstr):
                return optionstr
        configParser = BetterConfigParser()
        configParser.read("/etc/gdm/gdm.conf-custom")
        configParser.set("greeter", "GraphicalTheme", self.attDirname)
        configParser.set("greeter", "GraphicalThemes", "happygnome:/" + self.attDirname) # I'm not sure why this line is necessary
        if not os.path.exists("/etc/gdm/gdm.conf-custom.epidermisinit.bak"):
            shell.do(["cp", "/etc/gdm/gdm.conf-custom", "/etc/gdm/gdm.conf-custom.epidermisinit.bak"])
        import subprocess
        subprocess.call(["mkdir", "-p", "/tmp/epidermis"])
        subprocess.call(["touch", "/tmp/epidermis/gdm.conf-custom"])
        configParser.write(open("/tmp/epidermis/gdm.conf-custom", "w"))
        shell.do(["mv", "-f", "/tmp/epidermis/gdm.conf-custom", "/etc/gdm/gdm.conf-custom"])
        # Can't seem to get this to work with the shell, fortunately, it doesn't need to be run
        # as root, so I'll use subprocess
        subprocess.call("gdmflexiserver --command=\"UPDATE_CONFIG greeter/GraphicalTheme\"", shell=True)
        subprocess.call("gdmflexiserver --command=\"UPDATE_CONFIG greeter/GraphicalThemes\"", shell=True)

        progressCallback(1.0)
    
    def is_activated(self):
        """Returns whether this pigment is applied. To check if the effects
        have been activated regardless of Epidermis' pigments, use is_meat_activated"""
        return self.is_meat_activated()
    
    def is_attachment_correct(self, attachmentIndex=None):
        if not DirBasedPigment.is_attachment_correct(self, attachmentIndex):
            return False
        if attachmentIndex is None:
            att = self.attachments[0]
        else:
            att = self.attachments[attachmentIndex]
        if not os.path.exists(os.path.join(att, "GdmGreeterTheme.desktop")):
            return False
        return Pigment.is_attachment_correct(self, attachmentIndex)
    
    def is_meat_installed(self):
        """Returns whether the meat of this pigment or similar to it is installed"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        try:
            f1 = glob(self.attPath + "/*") 
            f2 = glob("/usr/share/gdm/themes/%s/*" % self.attDirname)
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            for af in ff:
                try:
                    ss = file(af, "rb").read(8096)
                except:
                    return False
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
        
    
    def is_meat_activated(self):
        """Returns whether the meat of this pigment or similar to it is activated"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        try:
            f1 = glob(self.attPath + "/*")
            from ConfigParser import SafeConfigParser # for reading INI files
            class BetterConfigParser(SafeConfigParser):
                """Better than SafeConfigParser: it's case-sensitive"""
                def __init__(self):
                    SafeConfigParser.__init__(self)
                def optionxform(self, optionstr):
                    return optionstr
            configParser = BetterConfigParser()
            configParser.read("/etc/gdm/gdm.conf-custom")
            nn = configParser.get("greeter", "GraphicalTheme")
            f2 = glob("/usr/share/gdm/themes/%s/*" % nn)
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            for af in ff:
                try:
                    ss = file(af, "rb").read(8096)
                except:
                    return False
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
    
    def generate_preview(self):
        if len(self.attPath) < 1:
            return None
        if not os.path.exists(os.path.join(self.attPath,  "GdmGreeterTheme.desktop")):
            return None
        desktopFile = open(os.path.join(self.attPath,  "GdmGreeterTheme.desktop"),  "r")
        screenshotName = None
        for line in desktopFile.readlines():
            line.strip()
            if line[:10] == "Screenshot" and line[10].strip()[0] == "=":
                screenshotName = line[len(line.split("=")[0]) + 1:].strip()
                break
        if screenshotName is None:
            return None
        try:
            pb = gtk.gdk.pixbuf_new_from_file_at_size(os.path.join(self.attPath, screenshotName),  100,  75)
        except:
            import traceback
            traceback.print_exc()
            return None
        if pb is None:
            return None
        else:
            self.preview = pb
            return pb
    
    def does_activation_require_root(self):
        """Returns whether the activation requires root rights.
        If it does, the pigment object will need a pexpect bash shell as root
        to be passed to some of its methods"""
        return True
    
    def get_display_data(self, index=0):
        """Returns the object of index to be added to a gtk.ListStore, this
        could be a string, a gtk.gdk.Pixbuf or a custom widget, it should
        be rendered using type.create_cell_renderer"""
        return

def main():
    wp = GDM()
    print wp.information_xml().toxml()

if __name__ == "__main__":
    main()
